package me.teridax.jcash.banking;

import me.teridax.jcash.decode.Decoder;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.*;

public final class BankingManagementSystem {

    private final Set<Bank> banks;

    private BankingManagementSystem() {
        this.banks = new HashSet<>();
    }

    private static String[] tail(String[] array, int index) {
        return Arrays.stream(array).skip(index).toArray(String[]::new);
    }

    public static BankingManagementSystem loadFromCsv(Path file) {
        try {
            var bms = new BankingManagementSystem();
            var content = Files.readString(file);

            content.lines().skip(1).forEach(line -> {
                var columns = line.split(";");

                if (columns.length != 14)
                    throw new IllegalArgumentException("invalid column count: " + columns.length);

                var owner = Owner.fromColumns(tail(columns, 8));
                var account = Account.fromColumns(tail(columns, 2));

                var blz = Bank.validateBlz(columns[1]);
                var name = Decoder.decodeName(columns[0]);
                var bankOfLine = new Bank(blz, name);

                var bankOfSet = bms.banks.stream().filter(b -> b.equals(bankOfLine)).findFirst();
                if (bankOfSet.isPresent()) {
                    bankOfSet.get().addAccount(owner, account);
                } else {
                    bankOfLine.addAccount(owner, account);
                    bms.banks.add(bankOfLine);
                }
            });

            return bms;

        } catch (IOException e) {
            throw new IllegalArgumentException("Could not read file " + file, e);
        } catch (IllegalArgumentException | NullPointerException e) {
            throw new IllegalArgumentException("Could not parse file " + file, e);
        }
    }
}
